package exnihilocreatio.client.renderers;

import exnihilocreatio.blocks.BlockInfestingLeaves;
import exnihilocreatio.client.models.ModelVertex;
import exnihilocreatio.texturing.Color;
import exnihilocreatio.tiles.TileInfestingLeaves;
import exnihilocreatio.util.Util;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.OpenGlHelper;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.vertex.VertexFormatElement;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.biome.BiomeColorHelper;
import net.minecraftforge.client.model.animation.FastTESR;

import javax.annotation.Nullable;


public class RenderInfestingLeaves extends FastTESR<TileInfestingLeaves> {

    private static final ModelVertex[] model = new ModelVertex[24];

    static {
        model[0] = new ModelVertex(EnumFacing.UP, 0, 1, 0, 0, 1);
        model[1] = new ModelVertex(EnumFacing.UP, 1, 1, 0, 1, 1);
        model[2] = new ModelVertex(EnumFacing.UP, 1, 1, 1, 1, 0);
        model[3] = new ModelVertex(EnumFacing.UP, 0, 1, 1, 0, 0);

        model[4] = new ModelVertex(EnumFacing.DOWN, 0, 0, 0, 0, 0);
        model[5] = new ModelVertex(EnumFacing.DOWN, 1, 0, 0, 1, 0);
        model[6] = new ModelVertex(EnumFacing.DOWN, 1, 0, 1, 1, 1);
        model[7] = new ModelVertex(EnumFacing.DOWN, 0, 0, 1, 0, 1);

        model[8] = new ModelVertex(EnumFacing.NORTH, 0, 0, 0, 1, 0);
        model[9] = new ModelVertex(EnumFacing.NORTH, 1, 0, 0, 0, 0);
        model[10] = new ModelVertex(EnumFacing.NORTH, 1, 1, 0, 0, 1);
        model[11] = new ModelVertex(EnumFacing.NORTH, 0, 1, 0, 1, 1);

        model[12] = new ModelVertex(EnumFacing.SOUTH, 0, 0, 1, 0, 0);
        model[13] = new ModelVertex(EnumFacing.SOUTH, 1, 0, 1, 1, 0);
        model[14] = new ModelVertex(EnumFacing.SOUTH, 1, 1, 1, 1, 1);
        model[15] = new ModelVertex(EnumFacing.SOUTH, 0, 1, 1, 0, 1);

        model[16] = new ModelVertex(EnumFacing.EAST, 1, 0, 0, 1, 0);
        model[17] = new ModelVertex(EnumFacing.EAST, 1, 0, 1, 0, 0);
        model[18] = new ModelVertex(EnumFacing.EAST, 1, 1, 1, 0, 1);
        model[19] = new ModelVertex(EnumFacing.EAST, 1, 1, 0, 1, 1);

        model[20] = new ModelVertex(EnumFacing.WEST, 0, 0, 0, 0, 0);
        model[21] = new ModelVertex(EnumFacing.WEST, 0, 0, 1, 1, 0);
        model[22] = new ModelVertex(EnumFacing.WEST, 0, 1, 1, 1, 1);
        model[23] = new ModelVertex(EnumFacing.WEST, 0, 1, 0, 0, 1);

    }

    @Override
    public void renderTileEntityFast(@Nullable TileInfestingLeaves te, double x, double y, double z, float partialTicks, int destroyStage, float partial, @Nullable BufferBuilder buffer) {
        if (te == null || buffer == null) return;
        final BlockPos pos = te.func_174877_v();
        final Block block = func_178459_a().func_180495_p(pos).func_177230_c();

        // Light levels
        final int mixedBrightness = func_178459_a().func_180495_p(pos).func_185889_a(te.func_145831_w(), pos);
        final int skyLight = mixedBrightness >> 16 & 0xFFFF;
        final int blockLight = mixedBrightness & 0xFFFF;
        //
        final IBlockState state = te.getLeafBlock();

        // Color
        final Color color;
        if (block instanceof BlockInfestingLeaves)
            color = Color.average(new Color(BiomeColorHelper.func_180287_b(func_178459_a(), pos)), Util.whiteColor, (float) Math.pow((te.getProgress() / 100f), 2.0));
        else
            color = Util.whiteColor;

        final TextureAtlasSprite sprite = Minecraft.func_71410_x().func_175602_ab().func_175023_a().func_178122_a(state);
        buffer.func_178969_c(x - pos.func_177958_n(), y - pos.func_177956_o(), z - pos.func_177952_p());
        OpenGlHelper.func_77475_a(OpenGlHelper.field_77476_b, skyLight * 16.0F, blockLight * 16.0F);

        Vec3d view = Minecraft.func_71410_x().func_175606_aa().func_174824_e(partialTicks);
        for (final ModelVertex vert : model) {
            final EnumFacing face = vert.face;

            // Dirty face culling
            double dot = 0;
            switch (face) {
                case DOWN:
                    dot = -(view.field_72448_b - (pos.func_177956_o()));
                    break;
                case UP:
                    dot = (view.field_72448_b - (pos.func_177956_o() + 1));
                    break;
                case NORTH:
                    dot = -(view.field_72449_c - (pos.func_177952_p()));
                    break;
                case SOUTH:
                    dot = (view.field_72449_c - (pos.func_177952_p() + 1));
                    break;
                case WEST:
                    dot = -(view.field_72450_a - (pos.func_177958_n()));
                    break;
                case EAST:
                    dot = (view.field_72450_a - (pos.func_177958_n() + 1));
                    break;
            }
            if (dot > 0 && block.func_176225_a(state, func_178459_a(), pos, vert.face)) {
                for (final VertexFormatElement e : buffer.func_178973_g().func_177343_g()) {
                    switch (e.func_177375_c()) {
                        case COLOR:
                            buffer.func_181666_a(color.r, color.g, color.b, color.a);
                            break;

                        case NORMAL:
                            buffer.func_181663_c(face.func_82601_c(), face.func_96559_d(), face.func_82599_e());
                            break;

                        case POSITION:
                            final double vertX = pos.func_177958_n() + vert.x;
                            final double vertY = pos.func_177956_o() + vert.y;
                            final double vertZ = pos.func_177952_p() + vert.z;

                            buffer.func_181662_b(vertX, vertY, vertZ);
                            break;

                        case UV:
                            if (e.func_177369_e() == 1) {
                                buffer.func_187314_a(skyLight, blockLight);
                            } else {
                                buffer.func_187315_a(sprite.func_94214_a(vert.u), sprite.func_94207_b(16.0 - vert.v));
                            }
                            break;

                        default:
                            break;
                    }
                }
                buffer.func_181675_d();
            }
        }
        // Don't screw everyone else up
        buffer.func_178969_c(0, 0, 0);
    }
}
