package exnihilocreatio.util;

import lombok.Getter;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.common.registry.ForgeRegistries;
import net.minecraftforge.oredict.OreDictionary;

import javax.annotation.Nonnull;

public class ItemInfo implements StackInfo {

    public static final ItemInfo EMPTY = new ItemInfo(ItemStack.field_190927_a);

    @Getter
    @Nonnull
    private Item item;

    private int meta = 0;

    @Getter
    private boolean isWildcard = false;

    public ItemInfo(@Nonnull Item item) {
        this.item = item;
        this.isWildcard = true;
    }

    public ItemInfo(@Nonnull Item item, int meta) {
        this.item = item;
        if (meta == -1 || meta == OreDictionary.WILDCARD_VALUE) {
            this.isWildcard = true;
        }
        else
            this.meta = meta;
    }

    public ItemInfo(@Nonnull ItemStack stack) {
        this.item = stack.func_77973_b();
        this.meta = stack.func_77952_i();
    }

    public ItemInfo(@Nonnull Block block) {
        this.item = Item.func_150898_a(block);
        this.meta = 0;
        this.isWildcard = true;
    }

    public ItemInfo(@Nonnull Block block, int blockMeta) {
        this.item = Item.func_150898_a(block);
        if (this.item == Items.field_190931_a){
            this.isWildcard = true;
        }
        else {
            if (blockMeta == -1 || blockMeta == OreDictionary.WILDCARD_VALUE) {
                this.isWildcard = true;
            } else
                this.meta = blockMeta;
        }
    }

    public ItemInfo(@Nonnull String string) {
        if (string.isEmpty() || string.length() < 2) {
            this.item = Items.field_190931_a;
            this.isWildcard = true;
            return;
        }
        String[] split = string.split(":");

        Item item = null;
        int meta = 0;

        switch (split.length) {
            case 1:
                item = Item.func_111206_d("minecraft:" + split[0]);
                break;
            case 2:
                try {
                    meta = split[1].equals("*") ? -1 : Integer.parseInt(split[1]);
                    item = Item.func_111206_d("minecraft:" + split[0]);
                } catch (NumberFormatException e) {
                    this.isWildcard = true;
                    meta = 0;
                    item = Item.func_111206_d(split[0] + ":" + split[1]);
                }
                break;
            case 3:
                try {
                    meta = split[2].equals("*") ? -1 : Integer.parseInt(split[2]);
                    item = Item.func_111206_d(split[0] + ":" + split[1]);
                } catch (NumberFormatException e) {
                    meta = 0;
                    this.isWildcard = true;
                }
                break;
            default:
                this.item = Items.field_190931_a;
                this.isWildcard = true;
                return;
        }

        if (item == null){
            this.item = Items.field_190931_a;
            this.isWildcard = true;
        }
        else {
            this.item = item;
            if (meta == -1 || meta == OreDictionary.WILDCARD_VALUE) {
                this.isWildcard = true;
            } else
                this.meta = meta;
        }
    }

    public ItemInfo(@Nonnull IBlockState state) {
        item = Item.func_150898_a(state.func_177230_c());
        meta = state.func_177230_c().func_176201_c(state);
    }

    public static ItemInfo readFromNBT(NBTTagCompound tag) {
        Item item = ForgeRegistries.ITEMS.getValue(new ResourceLocation(tag.func_74779_i("item")));
        int meta = tag.func_74762_e("meta");

        return item == null ? EMPTY : new ItemInfo(item, meta);
    }

    //StackInfo

    @Override
    public String toString() {
        return ForgeRegistries.ITEMS.getKey(item) + (meta == 0 ? "" : (":" + meta));
    }

    @Nonnull
    @Override
    public ItemStack getItemStack() {
        return item == Items.field_190931_a ? ItemStack.field_190927_a : new ItemStack(item, 1, meta);
    }

    @Override
    public boolean hasBlock() {
        return item instanceof ItemBlock;
    }

    @Nonnull
    @Override
    public Block getBlock() {
        return Block.func_149634_a(item);
    }

    @Nonnull
    @Override
    public IBlockState getBlockState() {
        if (item == Items.field_190931_a)
            return Blocks.field_150350_a.func_176223_P();
        try {
            return Block.func_149634_a(item).func_176203_a(meta);
        } catch (Exception e){
            return Block.func_149634_a(item).func_176223_P();
        }
    }

    @Override
    public int getMeta() {
        return isWildcard ? -1 : meta;
    }

    @Override
    public NBTTagCompound writeToNBT(NBTTagCompound tag) {
        tag.func_74778_a("item", ForgeRegistries.ITEMS.getKey(item) == null ? "" : ForgeRegistries.ITEMS.getKey(item).toString());
        tag.func_74768_a("meta", meta);

        return tag;
    }

    @Override
    public boolean isValid() {
        return this.item != Items.field_190931_a && meta <= OreDictionary.WILDCARD_VALUE;
    }

    @Override
    public int hashCode() {
        return this.item.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (isWildcard){
            if (obj instanceof ItemInfo)
                return ItemStack.func_185132_d(((ItemInfo) obj).getItemStack(), getItemStack());
            else if (obj instanceof ItemStack)
                return ItemStack.func_185132_d((ItemStack) obj, getItemStack());
            else if (obj instanceof BlockInfo)
                return ItemStack.func_185132_d(((BlockInfo) obj).getItemStack(), getItemStack());
            else if (obj instanceof Block) {
                BlockInfo block = new BlockInfo((Block) obj);
                return ItemStack.func_185132_d(block.getItemStack(), getItemStack());
            } else if (obj instanceof Item) {
                ItemInfo item = new ItemInfo((Item) obj);
                return ItemStack.func_185132_d(item.getItemStack(), getItemStack());
            }
        }
        else {
            if (obj instanceof ItemInfo)
                return ItemStack.func_77989_b(((ItemInfo) obj).getItemStack(), getItemStack());
            else if (obj instanceof ItemStack)
                return ItemStack.func_77989_b((ItemStack) obj, getItemStack());
            else if (obj instanceof BlockInfo)
                return ItemStack.func_77989_b(((BlockInfo) obj).getItemStack(), getItemStack());
            else if (obj instanceof Block) {
                BlockInfo block = new BlockInfo((Block) obj);
                return ItemStack.func_77989_b(block.getItemStack(), getItemStack());
            } else if (obj instanceof Item) {
                ItemInfo item = new ItemInfo((Item) obj);
                return ItemStack.func_77989_b(item.getItemStack(), getItemStack());
            }
        }
        return false;
    }
}
